/* 
 * Created : Sat Dec 21 00:43:53 CET 2002
 * Updated : Mon Jan 27 14:28:33 CET 2003
 *
 * Copyright (C) 2002-2003 sauron@kernsh.org
 * Copyright (C) 2002-2003 zorgon@kernsh.org
 *
 * THIS LICENCE IS ISSUED FROM NMAP LICENCE AND MODIFIED
 * 
 * The program kernsh is (C) 2002-2003 sauron/zorgon. This program is 
 * free software; you can redistribute it and/or modify it under the 
 * terms of the GNU General Public License as published by the Free
 * Software Foundation; Version 2. This guarantees your right to use,
 * modify, and redistribute this software under certain conditions. 
 * 
 * Source is provided to this software because we believe users have a
 * right to know exactly what a program is going to do before they run
 * it.                                                                  
 *
 * Source code also allows you to fix bugs, and add new features.
 * You are highly encouraged to send your changes to dev@kernsh.org for 
 * possible incorporation into the main distribution. By sending these 
 * changes to sauron@kernsh.org or zorgon@kernsh.org, it is assumed that 
 * you are offering them the unlimited, non-exclusive right to reuse, 
 * modify, and relicense the code. This is important because the 
 * inability to relicense code has caused devastating problems for other 
 * Free Software projects (such as KDE and NASM). Kernsh will always be 
 * available Open Source. If you wish to specify special license 
 * conditions of your contributions, just say so when you send them.
 *
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details (
 * http://www.gnu.org/copyleft/gpl.html ).
 *
 */

#include <linux/unistd.h>     /* les macros _syscallX */
#include "include/kernsh.h"

/* need by sysinfo() */
_syscall1(int, sysinfo, struct sysinfo *, info);

int
get_number_cpus( void )
{
	size_t cpus;

	cpus = get_symbol_by_name( "smp_num_cpus" );
	if ( !cpus ) {
		return( 1 );
	}

	/* I don't have a multi-processor system to test smp_num_cpus */
	return( 666 );
}

void
usage_display_system_info( void )
{
	fputs( 
	"display <no arguments>\n\n"
	"This command makes it possible to display informations about your system\n"
	"as for example uptime or cpus..\n\n", stdout );
	return;
}

#define KVBASE_MASK ( 0x1fffff )
void
do_display_system_info( int argc, char * argv[] )
{
	char * cmdline;
	struct utsname p_uname;
	struct sysinfo s_info;
	int size;
	int proc;
	
	uname( &p_uname );
	fprintf( stdout, "system name         : %s\n", p_uname.sysname );
	fprintf( stdout, "node name           : %s\n", p_uname.nodename );
	fprintf( stdout, "realease            : %s\n", p_uname.release );
	fprintf( stdout, "version             : %s\n", p_uname.version );
	fprintf( stdout, "machine name        : %s\n", p_uname.machine );
	fprintf( stdout, "domain name         : %s\n", 
	p_uname.__domainname );

	sysinfo( &s_info );
	fprintf( stdout, "uptime              : %lds\n", s_info.uptime );
	fprintf( stdout, "upload              : 1 min %ld / 5 min %ld / "
	"15 min %ld\n", s_info.loads[0], s_info.loads[1], s_info.loads[2] );
	fprintf( stdout, "RAM                 : total %ld / free %ld / "
	"shared %ld\n", s_info.totalram, s_info.freeram, s_info.sharedram );
	fprintf( stdout, "memory in buffers   : %ld\n", s_info.bufferram );
	fprintf( stdout, "swap                : total %ld / free %ld\n", 
	s_info.totalswap, s_info.freeswap );
	fprintf( stdout, "process number      : %d\n", s_info.procs );
	fprintf( stdout, "cpus                : %d\n", get_number_cpus() );
	fprintf( stdout, "page size           : %d\n", getpagesize() );
	fprintf( stdout, "kernel virtual base : %#x\n", 
	get_symbol_by_name("_stext") & ~KVBASE_MASK );

	fprintf( stdout, "system_call         : %#x\n", p_system_call );
	fprintf( stdout, "sys_call_table      : %#x\n", p_sys_call_table );
	fprintf( stdout, "idtr base           : %#x\n", (int)idtr.base );

	/* get boot arguments */
	cmdline = malloc( BUFSIZ );
	if ( cmdline == NULL ) {
		perror( "malloc()" );
		return;
	}

	/* we suppose that sizeof(boot arguments) < BUFSIZ */
	memset( cmdline, '\0', BUFSIZ );
#define CMDLINE "/proc/cmdline"
	fprintf( stdout, "boot arguments      : " );
	proc = open( CMDLINE, O_RDONLY );
	if ( proc < 0 ) {
		fatal( "( can't open %s file )\n", CMDLINE );
	}

	size = read( proc, cmdline, BUFSIZ );
	if ( size < 0 ) {
		perror( "read()" );
		return;
	}
	fprintf( stdout, "%s\n", cmdline );

	close( proc );
	return;
}	

