/* 
 * Created : Sat Dec 21 00:43:53 CET 2002
 * Updated : Thu Jan 30 01:16:11 CET 2003
 *
 * Copyright (C) 2002-2003 sauron@kernsh.org
 * Copyright (C) 2002-2003 zorgon@kernsh.org
 *
 * THIS LICENCE IS ISSUED FROM NMAP LICENCE AND MODIFIED
 * 
 * The program kernsh is (C) 2002-2003 sauron/zorgon. This program is 
 * free software; you can redistribute it and/or modify it under the 
 * terms of the GNU General Public License as published by the Free
 * Software Foundation; Version 2. This guarantees your right to use,
 * modify, and redistribute this software under certain conditions. 
 * 
 * Source is provided to this software because we believe users have a
 * right to know exactly what a program is going to do before they run
 * it.                                                                  
 *
 * Source code also allows you to fix bugs, and add new features.
 * You are highly encouraged to send your changes to dev@kernsh.org for 
 * possible incorporation into the main distribution. By sending these 
 * changes to sauron@kernsh.org or zorgon@kernsh.org, it is assumed that 
 * you are offering them the unlimited, non-exclusive right to reuse, 
 * modify, and relicense the code. This is important because the 
 * inability to relicense code has caused devastating problems for other 
 * Free Software projects (such as KDE and NASM). Kernsh will always be 
 * available Open Source. If you wish to specify special license 
 * conditions of your contributions, just say so when you send them.
 *
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details (
 * http://www.gnu.org/copyleft/gpl.html ).
 *
 */

#include "include/kernsh.h"

#define VERSION "0.2a-rc1"

size_t p_sys_call_table;
size_t p_system_call;
unsigned long sys_call_table[ NR_syscalls ];
int file_descriptor_kmem;
int modules_not_supported = 0;

/* generator function for command completion.  STATE lets us know 
 * whether to start from scratch; without any state (i.e. STATE == 0), 
 * then we start at the top of the list. 
 * http://www.ugcs.caltech.edu/info/readline/rlman_2.html
 */
char *
command_generator( char * text, int state )
{
    char * name;
	static int list_index;
	static int len;

	/* if this is a new word to complete, initialize now.  This 
	 * includes saving the length of TEXT for efficiency, and 
	 * initializing the index variable to 0. 
	 */
	if ( !state ) {
		list_index = 0;
		len = strlen( text );
	}

	/* return the next name which partially matches from the 
	 * command list. 
	 */
	while ( (name = p_command_main[list_index].name) ) {
		list_index++;
		if ( strncmp(name, text, len) == 0 ) {
			return( dupstr(name) );
		}
	}

	/* if no names matched, then return NULL. */
	return( (char *)NULL );
}

/* attempt to complete on the contents of TEXT.  START and END show the
 * region of TEXT that contains the word to complete.  We can use the
 * entire line in case we want to do some simple parsing.  Return the
 * array of matches, or NULL if there aren't any. 
 * http://www.ugcs.caltech.edu/info/readline/rlman_2.html
 */
char **
fileman_completion( char * text, int start, int end )
{
	char ** matches;

	matches = (char **)NULL;

	/* If this word is at the start of the line, then it is a command
	 * to complete.  Otherwise it is the name of a file in the current
	 * directory. 
	 */
	if ( start == 0 ) {
		matches = rl_completion_matches( text, 
		(rl_compentry_func_t *)command_generator );
	}

	return( matches );
}

void 
initialize_readline( void )
{
	/* allow conditional parsing of the ~/.inputrc file. */
	rl_readline_name = "kernsh";

	/* tell the completer that we want a crack first. */
	rl_attempted_completion_function = 
	(CPPFunction *)fileman_completion;
	return;
}

void 
do_exit( int argc, char * argv[] )
{
	exit( 0 );
}

#define ISBLANK( ch ) ( ((ch) == ' ') || ((ch) == '\t') )
int 
command( char * cmd )
{
	char command_name[ BUFSIZ ];
	char * p_command_name;
	char * argv[64];
	char * string;
	static const char delim[] = " \t\n";
	int argc = 0;
	int n;
	int i;
	int k;

	p_command_name = cmd;
	while ( *p_command_name && !ISBLANK(*p_command_name) ) {
		p_command_name++;
	}

	memcpy( command_name, cmd, p_command_name-cmd );
	command_name[p_command_name - cmd] = '\0';
	
	argv[argc++] = p_command_name;
	string = (char *)strtok( p_command_name, delim );
	while ( string != NULL ) {
		argv[argc++] = string;
		string = (char *)strtok( NULL, delim );
	}
	argv[argc] = '\0';

	for ( i = 0; i < sizeof(p_command_main)-1; i++ ) {
		if ( p_command_main[i].name == NULL ) {
			break;
		}

		n = strcmp( p_command_main[i].name, command_name );
		if ( n == 0 ) {
			break;
		}
	}

	if ( p_command_main[i].name == "help" ){
		if ( argc == 1 ) {
			for ( i = 0; i < sizeof(p_command_main)-1; 
			i++ ) {
        			if ( p_command_main[i].name == NULL ) {
					break;
				}

				if ( p_command_main[i].name != 
				"module" || (p_command_main[i].name == "module" 
				&& !modules_not_supported) ) {
					fprintf( stdout, "%s\t: %s\n", 
					p_command_main[i].name, 
					p_command_main[i].description );
				}
			}
			fputs( "\n", stdout );
			fputs( "Type \"help\" followed by command name "
			"for full documentation.\n\n", stdout );
			return( 0 );
		}
		else if ( argc == 2 ) {
			for ( k = 0; k < sizeof(p_command_main)-1; 
			k++ ) {
				if ( p_command_main[k].name == NULL ) {
					fatal( "undefined command..\n\n" );
					return( -1 );
				}

				if( !strcmp(p_command_main[k].name, argv[1])  ) {
					if(  p_command_main[k].usage != 0 ) {
						p_command_main[k].usage(); 
						return( 0 );
					} else {
						fatal( "no avaible help for this command\n\n" );
						return( -1 );
					}
				} else if ( strstr(p_command_main[k].name, argv[1]) ) {
					if ( p_command_main[k].usage != 0 ) {
						p_command_main[k].usage();
					}
				}
			}
		}
		else {
			fatal( "usage\n" );
			return( 0 );
		}
	}

	if ( p_command_main[i].name == NULL ) {
		fatal( "undefined command..\n" );
		return( -1 );
	}

	p_command_main[i].func( argc, argv );
	return( 0 );
}

void
initialization( int quiet )
{
	struct utsname p_uname;
	int offset;
	int n;

	if ( !quiet ) {
		uname( &p_uname );
		fprintf( stdout, "system name  : %s\n", p_uname.sysname );
		fprintf( stdout, "node name    : %s\n", p_uname.nodename );
		fprintf( stdout, "realease     : %s\n", p_uname.release );
		fprintf( stdout, "version      : %s\n", p_uname.version );
		fprintf( stdout, "machine name : %s\n", p_uname.machine );
		fprintf( stdout, "domain name  : %s\n\n", p_uname.__domainname );
	}

	/* open /dev/kmem */
	file_descriptor_kmem = open_kmem();
	if ( file_descriptor_kmem < 0 ) {
		fatal( "can't open /dev/kmem..\n" );
		exit( -1 );
	}

	/* we're checking integrity of system_call and sys_call_table 
	 * before to continue.
	 */

	/* get idt */
	asm( "sidt %0" : "=m" (idtr) );

	/* get system_call address */
	n = read_kmem( &idt, idtr.base + 8 * 0x80, sizeof(idt),
	file_descriptor_kmem );
	if ( n < 0 ) {
		exit( -1 );
	}
	p_system_call = (idt.off2 << 16) | idt.off1;
	
	if ( p_system_call != get_symbol_by_name("system_call") ) {
		fprintf( stdout, "\nthe IDT is hijacked ! fake system_call"
		" at %#x\n", p_system_call );
		exit( -1 );
	}

	/* get sys_call_table address */
	/* why +47 on a kernel 2.4.x and +48 on a kernel 2.2.x ? */
	offset = 47;
	if ( strstr(p_uname.release, "2.2") ) {
		offset = 48;
	}
	n = read_kmem( &p_sys_call_table, p_system_call+offset, 4, 
	file_descriptor_kmem );
	if ( n < 0 ) {
		exit( -1 );
	}

	if ( p_sys_call_table != get_symbol_by_name("sys_call_table") ) {
		fprintf( stdout, "\nsystem_call function hijacked ! "
		"new sys_call_table at %#x\n\n", p_sys_call_table );
		exit( -1 );
	}	

	/* get table of syscalls */
	n = read_kmem( &sys_call_table, p_sys_call_table, 
	sizeof(sys_call_table), file_descriptor_kmem );
	if ( n < 0 ) {
		exit( -1 );
	}

	return;
}	

int 
main( int argc, char * argv[] )
{
	FILE * fp = NULL;
	char * line;
	char * s;
	char c;
	uid_t uid;
	int fd;
	int quiet;
	
	/* test uid */
	uid = getuid(); 
	if ( uid ) { 
		fatal( "Hey ! You must be root\n" );
		return( -1 );
	}

	quiet = 0;
	while ( (c = getopt(argc, argv, "q")) != EOF ) {
		switch ( c ) {
			case 'q':
				quiet++;
				break;
			default:
				return( -1 );
		}
	}

	fprintf( stdout, "KERNSH - version %s <http://www.kernsh.org>\n", 
	VERSION ); 
	fputs( "Copyright (C) 2002-2003 sauron / zorgon\n\n", stdout );

	initialization( quiet );
	initialize_readline();

	/* test if modules are supported */
#define KSYMS "/proc/modules"
	fd = open( KSYMS, O_RDONLY );
	if ( fd < 0 ) {
		fatal( "[!] Modules aren't supported..\n\n" );
		modules_not_supported++;
	}

	for ( ;; ) {
		line = readline( PROMPT );		
		if ( !line ) {
			break;
		}

		/* remove leading and trailing whitespace from the line.
		 * then, if there is anything left, add it to the history list
		 * and execute it. 
		 */
		s = stripwhite( line );
		if ( *s ) {
			add_history( s );
			command( s );
		}
		//free( line );
	}

	if ( ferror(fp) ) {
		if ( fp == stdin ) {
			close_kmem( file_descriptor_kmem );
			return( -1 );
		}
	}

	clearerr( fp );
	close_kmem( file_descriptor_kmem );
	return( 0 );
}
