/* 
 * Created : Sat Dec 21 00:43:53 CET 2002
 * Updated : Thu Jan  2 10:56:54 CET 2003
 *
 * Copyright (C) 2002-2003 sauron@kernsh.org
 * Copyright (C) 2002-2003 zorgon@kernsh.org
 *
 * THIS LICENCE IS ISSUED FROM NMAP LICENCE AND MODIFIED
 * 
 * The program kernsh is (C) 2002-2003 sauron/zorgon. This program is 
 * free software; you can redistribute it and/or modify it under the 
 * terms of the GNU General Public License as published by the Free
 * Software Foundation; Version 2. This guarantees your right to use,
 * modify, and redistribute this software under certain conditions. 
 * 
 * Source is provided to this software because we believe users have a
 * right to know exactly what a program is going to do before they run
 * it.                                                                  
 *
 * Source code also allows you to fix bugs, and add new features.
 * You are highly encouraged to send your changes to dev@kernsh.org for 
 * possible incorporation into the main distribution. By sending these 
 * changes to sauron@kernsh.org or zorgon@kernsh.org, it is assumed that 
 * you are offering them the unlimited, non-exclusive right to reuse, 
 * modify, and relicense the code. This is important because the 
 * inability to relicense code has caused devastating problems for other 
 * Free Software projects (such as KDE and NASM). Kernsh will always be 
 * available Open Source. If you wish to specify special license 
 * conditions of your contributions, just say so when you send them.
 *
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details (
 * http://www.gnu.org/copyleft/gpl.html ).
 *
 */

#include "include/kernsh.h"

char *
dupstr( char * s )
{
	char * r;

	r = xmalloc( strlen(s)+1 );
	strcpy( r, s );
	return( r );
}

/* failure notice and exit */
void 
fatal( const char * format, ... ) 
{
	va_list ap;
	
	va_start( ap, format );
	vfprintf( stderr, format, ap );
	fflush( stderr );
	va_end( ap );
	return;
}

/* thanks kraken for hexdump function :-) */
void
hexdump( unsigned char * data, unsigned int amount )
{
	unsigned int dp;
	unsigned int p;
	const char trans[] =
	"................................ !\"#$%&'()*+,-./0123456789"
	":;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklm"
	"nopqrstuvwxyz{|}~...................................."
	"....................................................."
	"........................................";
	
	for ( dp = 1; dp <= amount; dp++ ) {
		printf( "%02x ", data[dp-1] );
		if ( (dp % 8) == 0 ) {
			printf( " " );
		}
		if ( (dp % 16) == 0 ) {
			printf( "| " );
			p = dp;
			for ( dp -= 16; dp < p; dp++ ) {
				printf( "%c", trans[data[dp]] );
			}
			printf( "\n" );
		}
	}

	if ( (amount % 16) != 0 ) {
		p = dp = 16 - ( amount % 16 );
		for ( dp = p; dp > 0; dp-- ) {
			printf( "   " );
			if ( ((dp % 8) == 0) && (p != 8) ) {
				printf( " " );
			}
		}
		printf( " | " );
		for ( dp = (amount - (16 - p)); dp < amount; dp++ ) {
			printf( "%c", trans[data[dp]] );
		}
	}
	printf( "\n" );
	return;
}

int
my_strncmp( char * s1, char * s2, int len )
{
    int u;

    for ( u = 0; u != len; u++ ) {
        if ( *(s1+u) != *(s2+u) ) {
            return( -1 );
        }
    }
    return( 0 );
}

int
foundnext( char * s1, char c, int len )
{
    int i;

    for ( i = 0; i < len; i++ ) {
        if ( *(s1+i) == c ) {
            return( i );
        }
    }
    return( 0 );
}

int
parse( char * s1, char * s2, size_t addr )
{
	int tot; 
	int len; 
	int max;
	int l;
	int parse;

	parse = tot = 0;

	for ( max = BUFSIZ, l = strlen(s2), len = tot = 0; max > 0; ) {
        tot = foundnext( s1, * s2, max );
        if ( tot ) {
            if ( my_strncmp(&s1[tot], s2, l) == 0 ) {
			fprintf( stdout, "pattern found at %#x\n",
                        (size_t)(addr+len+tot) );
			fflush( stdout );
			parse = 1;
            }
            s1 += tot + l;
            max -= tot + l;
            len += tot + l;
        }
        else {
            break;
        }
    }
    return( parse );
}

char
enter( void )
{
    fputs( "---Type <return> to continue, or q <return> to quit---\n", 
	stdout );
    return( getchar() );
}

/* strip whitespace from the start and end of STRING.  Return a pointer
 * into STRING. 
 */
char *
stripwhite( char * string )
{
	register char * s; 
	register char * t;

	for ( s = string; whitespace(*s); s++ );
	if ( *s == 0 ) {
		return( s );
	}

	t = s + strlen( s ) - 1;
	while ( t > s && whitespace(*t) ) {
		t--;
	}
	*++t = '\0';
	return( s );
}

char *
string2hex( char * str, int * nu )
{
	char * pointer;
	char tmp[ 5 ];
	int i;
	int a;
	int len;

	len = strlen( str );
	a = len / 4;
	*nu = a;

	pointer = malloc( a+1 );
	if ( pointer == NULL ) {
		perror( "malloc()" );
		return( NULL );
	}
	memset( pointer, '\0', a+1 );

	for ( i = a = 0; i < len; i += 4 ) {
		if ( str[i] == '\\' ) {
			strcpy( tmp, "0x" );
			strncat( tmp, &str[i+2], 2 );
			tmp[ sizeof(tmp)-1 ] = 0;
			pointer[a] = strtoul( tmp, NULL, 16 );
			a++;
			if ( a == len/4 ) {
				break;
			}
		}
	}
	
	if ( !a ) {
		fatal( "string %s not valid..\n\n", str );
		return( NULL );
	}
	return( pointer );
}

void *
xmalloc( size_t size )
{
	char * buf;

	buf = malloc( size );
	if ( buf == NULL ) {
		return( NULL );
	}

	memset( buf, 0, size );
	return( buf );
}

