/* 
 * Created : Sat Dec 21 00:43:53 CET 2002
 * Updated : Thu Jan  2 10:56:54 CET 2003
 *
 * Copyright (C) 2002-2003 sauron@kernsh.org
 * Copyright (C) 2002-2003 zorgon@kernsh.org
 *
 * THIS LICENCE IS ISSUED FROM NMAP LICENCE AND MODIFIED
 * 
 * The program kernsh is (C) 2002-2003 sauron/zorgon. This program is 
 * free software; you can redistribute it and/or modify it under the 
 * terms of the GNU General Public License as published by the Free
 * Software Foundation; Version 2. This guarantees your right to use,
 * modify, and redistribute this software under certain conditions. 
 * 
 * Source is provided to this software because we believe users have a
 * right to know exactly what a program is going to do before they run
 * it.                                                                  
 *
 * Source code also allows you to fix bugs, and add new features.
 * You are highly encouraged to send your changes to dev@kernsh.org for 
 * possible incorporation into the main distribution. By sending these 
 * changes to sauron@kernsh.org or zorgon@kernsh.org, it is assumed that 
 * you are offering them the unlimited, non-exclusive right to reuse, 
 * modify, and relicense the code. This is important because the 
 * inability to relicense code has caused devastating problems for other 
 * Free Software projects (such as KDE and NASM). Kernsh will always be 
 * available Open Source. If you wish to specify special license 
 * conditions of your contributions, just say so when you send them.
 *
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details (
 * http://www.gnu.org/copyleft/gpl.html ).
 *
 */

#include "include/kernsh.h"
#include "include/sysc.h"

void
get_list_addr_syscall( void )
{
	int i;

	for ( i = 0; i < NR_syscalls; i=i+2 ) {
		fprintf( stdout, "syscall# %3i: 0x%lx\tsyscall# %3i: 0x%lx\n", 
		i, (unsigned long)sys_call_table[i], i+1, 
		(unsigned long)sys_call_table[i+1] );
	}
	fputs( "\n", stdout );
	return;
}

void
call_syscall( int number )
{
	long __res;

	__asm__ volatile( "int $0x80"
		: "=a" (__res)
		: "0" (number) );
	return;
}

void
hijack_syscall( int number, size_t kaddr )
{
	int n;

	n = write_kmem( &kaddr, p_sys_call_table+(sizeof(void *)*number),
	sizeof(kaddr), file_descriptor_kmem );
	if ( n < 0 ) {
		fatal( "writing in %s failed !\n", KMEM );
	}

	/* sys_call_table is modified, so we initialize again 
	 * sys_call_table 
	 */
	n = read_kmem( &sys_call_table, p_sys_call_table,
	sizeof(sys_call_table), file_descriptor_kmem );
	if ( n < 0 ) {
		return;
	}
	return;
}

/* That is not enough to check the first 7 bits representing a jmp. 
 * I thus prefer to make a file of signatures of the syscalls on a 
 * healthy system and to make a checking starting from this check.
 */
void
signatures_syscall( char * signatures_file, int size_sign )
{
#define CREATE_SYSCALL "signature of syscalls\n"
	create_file_signatures( CREATE_SYSCALL, signatures_file, size_sign,
	SIG_FOR_SYSCALL );
	return;
}

void
check_syscall( char * signatures_file, int size_sign )
{
	int n;

#define CHECK_SYSCALL "\nn\tsignature\t\tactual\n" \
	"----------------------------------------------------------\n"
	n = check_file_signatures( CHECK_SYSCALL, signatures_file, size_sign,
	SIG_FOR_SYSCALL );
	if ( n == EXIT_SUCCESS ) {
		fputs( "syscall_table valid..\n\n", stdout );
	}
	return;
}


void
get_definition_syscall( int nr )
{
	size_t start_address;
	size_t end_address;
	char c;
	int n;

	fputs( "         name : ", stdout );
	if ( !strlen(list_syscall[nr]) ) {
		fputs( "(null)\n", stdout );
	}
	else {
		fprintf( stdout, "%s\n", list_syscall[nr] );
	}
	
	fprintf( stdout, "   syscall n : %d\n", nr );
	start_address = (unsigned int)sys_call_table[nr];
	fprintf( stdout, "start address : %#x\n", start_address );

	end_address = get_next_symbol_by_addr( start_address );
	if ( end_address != 0 ) {
		fprintf( stdout, "  end address : %#x\n", end_address-1 );
		fprintf( stdout, "       length : %d\n", 
		end_address-start_address-1 );
		fputs( "\n", stdout );

		fputs( "[!] opcode or disass. code [o/d] ?", stdout );
		c = getchar();
		if ( c == 'o' ) {
			/* \n */
			getchar();
			fprintf( stdout, "opcode from address %#x..\n", 
			start_address );
			kdump( start_address, end_address-start_address-1, 
			file_descriptor_kmem );
		}
		else if ( c == 'd' ) {
			/* \n */
			getchar();
			fprintf( stdout, "[!] disassembling from address %#x..\n",
			start_address );
			n = disasm( start_address, end_address-start_address-1 );
			if ( n < 0 ) {
				fatal( "can't disasm this syscall..\n" );
			}
		}
		else {
			fatal( "[!] bad choice sorry..\n" );
		}
	}
	fputs( "\n", stdout );
	return;
}

void
get_definition_syscall_by_number( int nr )
{
	get_definition_syscall( nr );
}

void
get_definition_syscall_by_addr( char * address )
{
	size_t p_syscall;
	int i;
	int n;
	int nr;

	p_syscall = strtoul( address, NULL, 16 );
	if ( p_syscall < 0xc0000000 || p_syscall > 0xffffffff ) {
		fatal( "address not valid..\n" );
		return;
	}

	for ( i = 0, n = 0; i < NR_syscalls; i++ ) {
		if ( p_syscall == (unsigned int)sys_call_table[i] ) {
			nr = i;
			n++;
			if ( n > 1 ) {
				fprintf( stdout, "address of syscall "
				"sys_ni_syscall..\n\n" );
				return;
			}
		}
	}
    
	if ( nr ) {
		get_definition_syscall( nr );
	}
	else {
		fatal( "syscall at %#x don't exist..\n\n", p_syscall );
	}
	return;
}

void
get_definition_syscall_by_name( char * name_syscall )
{
	int i;
	int nr;

	nr = 0;
	for ( i = 0; i < NR_syscalls; i++ ) {
		if ( !strcmp(list_syscall[i], name_syscall) ) {
			nr = i;
		}
	}

	if ( nr ) {
		get_definition_syscall( nr );
	}
	else {
		fatal( "syscall %s don't exist..\n\n", name_syscall );
	}	
	return;
}

void
usage_syscall( void )
{
	fputs( 
	"syscall [ -dangshlc ] <arguments>\n\n"
	"This command makes it possible to play with the syscalls:\n"
	"list syscalls and their addresses, syscall hijack..\n\n"
	"__options\n"
	"  -d [syscall number] : display characteristics of a syscall according\n" 
	"to a number ;\n" 
	"  -a [address] : display characteristics of a syscall according to\n"
	"an address ;\n"
	"  -n <syscall name> : display characteristics of a syscall according\n"
	"to a name ;\n"
	"  -g [filename] : create a file of signatures ;\n"
	"  -s [filename] : check syscall_table integrity ;\n"
	"  -h [syscall number] <syscall address> : hijack a syscall ;\n"
	"  -l : list syscalls and theirs addresses ;\n"
	"  -c [syscall number] : call a syscall according to a number ;\n\n",
	stdout );
	return;
}

#define HIJACK_SYSCALL 0
#define CREATE_SIGNATURE_SYSCALL 1
#define CHECK_SIGNATURE_SYSCALL 2

void
do_syscall( int pargc, char * pargv[] )
{
	char c;
	size_t l;
	int n;
	int flag;
	int size_sign;

	NOARGUMENTS( pargc, 1, syscall );

	optind = 0;
	flag = -1;

	while ( (c = getopt(pargc, pargv, "hls:g:d:c:a:n:")) != EOF ) {
		switch ( c ) {
			case 'l':
				get_list_addr_syscall();
				break;
			case 'c':
				n = atoi( optarg );
				CHECK_NUMBER_SYSCALL( n );
				call_syscall( n );
				break;
			case 's':
				flag = CHECK_SIGNATURE_SYSCALL;
				break;
			case 'g':
				flag = CREATE_SIGNATURE_SYSCALL;
				break;
			case 'h':
				flag = HIJACK_SYSCALL;
				break;
			case 'd':
				n = atoi( optarg );
				CHECK_NUMBER_SYSCALL( n );
				get_definition_syscall_by_number( n );
				break;
			case 'a':
				get_definition_syscall_by_addr( optarg );
				break;
			case 'n':
				get_definition_syscall_by_name( optarg );
				break;
			default:
				usage_syscall();
				return;
		}
	}

	switch ( flag ) {
		case HIJACK_SYSCALL:
			if ( pargc != 4 ) { 
				usage_syscall();
				return;
			}
			n = atoi( pargv[2] );
			CHECK_NUMBER_SYSCALL( n );
			XSTRTOUL( pargv[3], l );

			fprintf( stdout, "hijacked syscall : %d\n", n );
			fprintf( stdout, "    with address : %#x\n", l );
			fputs( "\n", stdout );
			hijack_syscall( n, l );
			break;

		case CHECK_SIGNATURE_SYSCALL:
			if ( pargc < 3 ) {
				usage_idt();
				return;
			}

			size_sign = 4;
			if ( pargc == 4 ) {
				size_sign = atoi( pargv[3] );
				if ( size_sign > 64 ) {
					fatal( "please, choose size < 64..\n" );
					return;
				}
			}
			check_syscall( pargv[2], size_sign );
			break;

		case CREATE_SIGNATURE_SYSCALL:
			if ( pargc < 3 ) {
				usage_idt();
				return;
			}

			size_sign = 4;
			if ( pargc == 4 ) {
				size_sign = atoi( pargv[3] );
				if ( size_sign > 64 ) {
					fatal( "please, choose size < 64..\n" );
					return;
				}
			}
			signatures_syscall( pargv[2], size_sign );
			break;

		default:
			return;
	}
	return;
}
